#------------------- Code to Support Chapter 2 --------------------#

#*************************************** Note regarding estimation method **********************************#
# This code is written to analyze the CESD data in TAM using Joint Maximum Likelihood Estimation (JMLE).
# JMLE was selected to match the results from Facets and Winsteps as closely as possible.
# Other estimation methods are available in TAM. Readers who prefer those methods may modify the code as 
# needed to use them, as described in the package documentation.
#************************************************************************************************************#

########## Prepare for the Analysis ###############

# Install and load the TAM package:
install.packages("TAM")
library("TAM")

# Install and load the Wright Map package:
install.packages("WrightMap")
library(WrightMap)

# Import the data:

## Note: Set the working directory to the folder where you have saved the data.
## Then, you should be able to import it using the code below:
cesd <- read.csv("Simulated_CESD_with_demographics.csv")

# Isolate the response matrix for analysis:
cesd.responses <- subset(cesd, select = c(item1:item20))


############# Rating Scale Model (RSM) ################

# Model specification:

## Specify the design matrix for the model:
design.matrix <- designMatrices(resp=cesd.responses, modeltype="RSM", constraint = "items")$A

## Run the RSM using the design matrix
RSM.CESD_JMLE <- tam.jml(cesd.responses, A = design.matrix, constraint = "items",
                            control=list(maxiter=500), version=2)

### Request a summary of the model results
summary(RSM.CESD_JMLE)

# Item results:

## Extract the item location estimates from the model
items_JMLE <- RSM.CESD_JMLE$xsi[c(1:ncol(cesd.responses))]
items_JMLE

## Examine Threshold Parameters:

### Specify the number of thresholds as the maximum observed score in the response matrix (be sure the responses begin at category 0):
n.thresholds <- max(cesd.responses)

### Calculate adjacent-category threshold values:
tau.estimates_JMLE <- NULL

for(tau in 1: (n.thresholds - 1)){
  tau.estimates_JMLE[tau] <- RSM.CESD_JMLE$item1$xsi[ncol(cesd.responses) -1 + tau ]
}
tau.estimates_JMLE[n.thresholds] <- -(sum(tau.estimates_JMLE))

### Print adjacent-categories threshold estimates to the console:
tau.estimates_JMLE


#  Evaluate Model Requirements

## Evaluate unidimensionality

### Extract model residuals:
resids <- IRT.residuals(RSM.CESD_JMLE)

### Find expected responses:
exp <- resids$X_exp

### Calculate raw (unstandardized) residuals:
resids.raw <- as.matrix(cesd.responses - exp)

### Calculate the variance in observations due to Rasch-model-estimated locations:

#### Variance of the observations: VO
observations.vector <- as.vector(as.matrix(cesd.responses))
VO <- var(observations.vector)
VO

#### Variance of the residuals: VR
residuals.vector <- as.vector(resids.raw)
VR <- var(residuals.vector)
VR

#### Raw variance explained by Rasch measures: (VO - VR)/VO
(VO - VR)/VO

#### Express the result as a percent:
((VO - VR)/VO) * 100

## Evaluate local item independence with inter-item standardized residual correlations:
resid.corrs <- as.data.frame(cor(resids$stand_residuals))

select_all_but_diag <- function(x) matrix(x[lower.tri(x, diag = F) | upper.tri(x, diag = F)], nrow = nrow(x) - 1, ncol = ncol(x))

### Find the maximum inter-item standardized residual corelation:
resid.corrs.unique <- select_all_but_diag(resid.corrs)

max(resid.corrs.unique)


#  Item Fit Analysis

## Calculate numeric item fit statistics
JMLE_fit <- tam.fit(RSM.CESD_JMLE)
item.fit_JMLE <- JMLE_fit$fit.item

### View summary statistics for item fit statistics
summary(item.fit_JMLE)

# Person Parameters

## Store person parameters and their standard errors in a data frame object:
person.locations_JMLE <- cbind.data.frame(RSM.CESD_JMLE$theta,
                                          RSM.CESD_JMLE$errorWLE)
names(person.locations_JMLE) <- c("theta", "SE")

### View summary statistics for person parameters:
summary(person.locations_JMLE)


# Person Fit 

## Examine numeric person fit indices
JMLE_fit <- tam.fit(RSM.CESD_JMLE)

### View summary statistics for the fit statistics.
person.fit_JMLE <- JMLE_fit$fit.person
summary(person.fit_JMLE)

# Wright Map

## Prepare estimates for plotting:

### Combine item estimates with thresholds:

n.items <- ncol(cesd.responses)

thresholds_JMLE <- matrix(data = NA, nrow = n.items, ncol = n.thresholds)
for(i in 1:n.thresholds){
  thresholds_JMLE[, i] <- items_JMLE + tau.estimates_JMLE[i]
}


## Plot the Wright Map

wrightMap(thetas = person.locations_JMLE$theta,
          thresholds = thresholds_JMLE,
          main.title = "Rating Scale Model Wright Map: CESD",
          show.thr.lab	= TRUE, dim.names = "",
          label.items.rows= 2)

#* Note: Sometimes, plots that are constructed after a WrightMap() object are distorted.
#* Using the graphics.off() function seems to resolve this issue. 
#* However, note that it will clear plots from the R Studio environment.
graphics.off()



#################### Partial Credit Model (PCM) ###################

## Specify the design matrix for the model:
design.matrix <- designMatrices(resp=cesd.responses, modeltype="PCM", constraint = "items")$A

## Run the PCM using the design matrix
PCM.CESD_JMLE <- tam.jml(cesd.responses, A = design.matrix, constraint = "items",
                         control=list(maxiter=500), version=2 )

### Request a summary of the model results
summary(PCM.CESD_JMLE)

## Examine Item Parameters:

### Extract the item location estimates from the model
items_JMLE <- PCM.CESD_JMLE$item$xsi.item

### Center the item parameter estimates for ease of interpretation
uncentered.item.locations_JMLE <- items_JMLE
centered.item.locations_JMLE <- scale(uncentered.item.locations_JMLE, scale = FALSE)

### Request summary statistics for the centered item estimates
summary(centered.item.locations_JMLE)

## Examine Threshold Parameters:
n.thresholds <- max(cesd.responses)

### Save the threshold estimates, which begin in column 5 of the item table:

tau.estimates_JMLE <- PCM.CESD_JMLE$item[, c(5 : (4+n.thresholds))]

### Specify the number of items that were included in the analysis:
n.items <- ncol(cesd.responses)

### Specify the number of thresholds as the maximum observed score in the response matrix (be sure the responses begin at category 0):
n.thresholds <- max(cesd.responses)

### Create a matrix in which to store the adjacent-category threshold values for each item:
tau.matrix_JMLE <- matrix(data = NA, ncol = n.thresholds, nrow = n.items)

### Calculate adjacent-category threshold values:

for(item.number in 1:n.items){
  for(tau in 1:n.thresholds){
    tau.matrix_JMLE[item.number, tau] <- ifelse(tau == 1,
                                                (tau.estimates_JMLE[item.number, tau] -  
                                                   uncentered.item.locations_JMLE[item.number]),
                                                
                                                (tau.estimates_JMLE[item.number, tau] -
                                                   sum(tau.estimates_JMLE[item.number, c((tau-1))]) - 
                                                   uncentered.item.locations_JMLE[item.number]))
  }
}


### View the threshold estimates:
tau.matrix_JMLE

## Model-Data Fit

## Evaluate unidimensionality

### Extract model residuals:

resids <- IRT.residuals(PCM.CESD_JMLE)

### Find expected responses:
exp <- resids$X_exp

## Calculate raw (unstandardized) residuals:
resids.raw <- as.matrix(cesd.responses - exp)

### Calculate the variance in observations due to Rasch-model-estimated locations:

##### Variance of the observations: VO
observations.vector <- as.vector(as.matrix(cesd.responses))
VO <- var(observations.vector)
VO

##### Variance of the residuals: VR
residuals.vector <- as.vector(resids.raw)
VR <- var(residuals.vector)
VR

##### Raw variance explained by Rasch measures: (VO - VR)/VO
(VO - VR)/VO

#### Express the result as a percent:
((VO - VR)/VO) * 100

## Evaluate local item independence with inter-item standardized residual correlations:

resid.corrs <- as.data.frame(cor(resids$stand_residuals))

select_all_but_diag <- function(x) matrix(x[lower.tri(x, diag = F) | upper.tri(x, diag = F)], nrow = nrow(x) - 1, ncol = ncol(x))

resid.corrs.unique <- select_all_but_diag(resid.corrs)
max(resid.corrs.unique)


## Numeric item fit statistics
JMLE_fit <- tam.fit(PCM.CESD_JMLE)
item.fit_JMLE <- JMLE_fit$fit.item

### View summary statistics for item fit statistics
summary(item.fit_JMLE)

## Examine person parameters

### Store person parameters and their standard errors in a dataframe object:
person.locations_JMLE <- cbind.data.frame(PCM.CESD_JMLE$theta,
                                          PCM.CESD_JMLE$errorWLE)

names(person.locations_JMLE) <- c("theta", "SE")

# View summary statistics for person parameters:
summary(person.locations_JMLE)

# Subtract the original (uncentered) item mean location from the person locations:
person.locations_JMLE$theta_adjusted <- person.locations_JMLE$theta - mean(uncentered.item.locations_JMLE)

# Summary of person location estimates:
summary(person.locations_JMLE)

## Person Fit:

# Examine numeric person fit indices
JMLE_fit <- tam.fit(PCM.CESD_JMLE)

# View summary statistics for the fit statistics.
person.fit_JMLE <- JMLE_fit$fit.person
summary(person.fit_JMLE)

## Wright Map:

### Prepare estimates for plotting:

#### Combine centered item estimates with thresholds:
n.items <- ncol(cesd.responses)

thresholds_JMLE <- matrix(data = NA, nrow = n.items, ncol = n.thresholds)

for(item.number in 1:n.items){
  for(tau in 1:n.thresholds){
    thresholds_JMLE[item.number, tau] <- centered.item.locations_JMLE[item.number] +
      tau.matrix_JMLE[item.number, tau]
  } 
}

thetas_JMLE <- person.locations_JMLE$theta_adjusted

# Plot the Variable Map
wrightMap(thetas = thetas_JMLE,
          thresholds = thresholds_JMLE,
          main.title = "Partial Credit Model Wright Map: CESD",
          show.thr.lab	= TRUE, dim.names = "",
          label.items.rows= 2)

#* Note: Sometimes, plots that are constructed after a WrightMap() object are distorted.
#* Using graphics.off() seems to resolve this issue.
graphics.off()





######### Partial-Credit Many-Facet Rasch Model ##########

# Model specification:

## specify facets that will be included in the model
### (Note that items are a facet by default)

facets <- cesd[ , "education", drop=FALSE] 

## Specify person identification numbers:
pid <- cesd$id 

## Specify the response matrix
resp <- cesd.responses

## Specify the model formula:
PC_MFR_formula <- ~ item + education + (step+education:step)


# Run the PC-MFR model:
PC_MFR_model <- tam.mml.mfr(resp= resp, facets = facets,
                                     formulaA= PC_MFR_formula,
                            pid= pid, verbose = FALSE, 
                            constraint = "items")

## Request a summary of the model results:
summary(PC_MFR_model)

### Examine Facet Estimates:

facet.estimates <- PC_MFR_model$xsi.facets # all facets together

item.estimates <- subset(facet.estimates, facet.estimates$facet == "item")

education.estimates <- subset(facet.estimates, facet.estimates$facet == "education")

threshold.estimates <- subset(facet.estimates, facet.estimates$facet == "step")

education_threshold.estimates <- subset(facet.estimates, facet.estimates$facet == "step:education")


### Examine item locations:

summary(item.estimates$xsi)
hist(item.estimates$xsi, xlab = "Logits", col = "darkseagreen3", main = "Item Location Estimates")

### Examine education locations:

summary(education.estimates$xsi)
plot(education.estimates$xsi, main = "Education Subgroup Locations",
     xlab = "Education Subgroup Number", ylab = "Location Estimate", pch = 18,
     col = "darkseagreen3")

### Examine person locations:

person.estimates <- tam.wle(PC_MFR_model)
theta <- person.estimates$theta
summary(theta)
hist(theta, xlab = "Logits", col = "darkseagreen3", main = "Person Location Estimates")


## Model-Data Fit

## Evaluate unidimensionality

### Extract model residuals:
resids <- IRT.residuals(PC_MFR_model)

#* Note: With the MFR model, residuals are presented separately for each level of the explanatory facets.
#* We need to do some manipulation to construct a typical residual matrix.

### Calculate raw residuals and store them in a matrix:
r <- as.data.frame(resids$residuals)

resid.matrix <- matrix(data = NA, nrow = nrow(cesd.responses), ncol = ncol(cesd.responses))

ngroups <- length(unique(cesd$education))

for(item.number in 1:ncol(cesd.responses)){
  group.items <- NULL
  
  for(group in 1:ngroups){
    group.items[group] <- paste("item", item.number, "-", "education", group, sep = "")
  }
  
  item <- subset(r, select = group.items)

  resid.matrix[, item.number] <- rowSums(item, na.rm = TRUE)
}


### Store standardized residuals in a matrix:

s <- as.data.frame(resids$stand_residuals)

std.resid.matrix <- matrix(data = NA, nrow = nrow(cesd.responses), ncol = ncol(cesd.responses))

ngroups <- length(unique(cesd$education))

for(item.number in 1:ncol(cesd.responses)){
  group.items <- NULL
  
  for(group in 1:ngroups){
    group.items[group] <- paste("item", item.number, "-", "education", group, sep = "")
  }
  
  item <- subset(s, select = group.items)
  
  std.resid.matrix[, item.number] <- rowSums(item, na.rm = TRUE)
}

### Calculate the variance in observations due to Rasch-model-estimated locations:

##### Variance of the observations: VO
observations.vector <- as.vector(as.matrix(cesd.responses))
VO <- var(observations.vector)
VO

##### Variance of the residuals: VR
residuals.vector <- as.vector(resid.matrix)
VR <- var(residuals.vector)
VR

##### Raw variance explained by Rasch measures: (VO - VR)/VO
(VO - VR)/VO

#### Express the result as a percent:
((VO - VR)/VO) * 100

## Evaluate local item independence with inter-item standardized residual correlations:

resid.corrs <- as.data.frame(cor(std.resid.matrix))

select_all_but_diag <- function(x) matrix(x[lower.tri(x, diag = F) | upper.tri(x, diag = F)], nrow = nrow(x) - 1, ncol = ncol(x))

resid.corrs.unique <- select_all_but_diag(resid.corrs)

### Find the maximum correlation between inter-item standardized residuals:
max(resid.corrs.unique)


## Numeric item fit statistics

### Note that for the MFR model, fit is shown for item*subgroup combinations
item.fit <- msq.itemfit(PC_MFR_model)

#### summary of item fit:
item.fit$summary_itemfit

### detailed item fit results:
item.fit.results <- as.data.frame(item.fit$itemfit)
item.fit.results

## Examine person fit:
person.fit <- tam.personfit(PC_MFR_model)

### summary of person fit:
summary(person.fit)

## Plot a Wright Map

### Create a matrix in which to store threshold estimates for each education group:
ngroups <- length(unique(cesd$education))
n.thresholds <- max(cesd.responses)
group_taus <- matrix(data = NA, nrow = ngroups, ncol = n.thresholds)

### re-structure the threshold estimates so that they can be added to the matrix:
all_group_taus <- t(education_threshold.estimates$xsi)

### Use a for-loop to add the results to the matrix. 

#* Note: For group 1, the thresholds for the first group are equal to the overall 
#* threshold estimates from the model. We will store the group-specific threshold
#* locations as zero logits for now, and then add the threshold values later.

group_taus[1, ] <- rep(0, n.thresholds)

for(group.number in 2:ngroups){
  group_taus[group.number,] <- all_group_taus[c((group.number * n.thresholds) - (n.thresholds - 1)) :
                                                (group.number * n.thresholds)]
}

### convert the matrix to a data frame for easier manipulation and labeling:
group_taus <- as.data.frame(group_taus)

### label the education-specific threshold estimates:
names(group_taus) <- c("t1", "t2", "t3")

### add overall threshold estimates:
group_taus$t1 <- group_taus$t1 + threshold.estimates$xsi[1]
group_taus$t2 <- group_taus$t2 + threshold.estimates$xsi[2]
group_taus$t3 <- group_taus$t3 + threshold.estimates$xsi[3]

### Plot the Wright map:

wrightMap(thetas = person.estimates$theta,
          thresholds = group_taus,
          main.title = "Partial Credit Many-Facet Rasch Model Wright Map: CESD",
          show.thr.lab	= TRUE, dim.names = "",
          label.items.rows= 2, label.items = c("1", "2", "3", "4", "5", "6"),
          axis.items = "Education Subgroups")

