#------------------- Code to Support Chapter 3 --------------------#

#*************************************** Note regarding estimation method **********************************#
# This code is written to analyze the CESD data in TAM using Joint Maximum Likelihood Estimation (JMLE).
# JMLE was selected to match the results from Facets and Winsteps as closely as possible.
# Other estimation methods are available in TAM. Readers who prefer those methods may modify the code as 
# needed to use them, as described in the package documentation.
#************************************************************************************************************#

########## Prepare for the Analysis ###############

# Install and load the TAM package:
install.packages("TAM")
library("TAM")

# Install and load the Wright Map package:
install.packages("WrightMap")
library(WrightMap)

# Install and load the plyr package:
install.packages("plyr")
library(plyr)

# Import the data:

#* Note: Set the working directory to the folder where you have saved the data.
#* Then, you should be able to import it using the code below:
cesd <- read.csv("Simulated_CESD_with_demographics.csv")

# Isolate the response matrix for analysis:
cesd.responses <- subset(cesd, select = c(item1:item20))


############# Rating Scale Model (RSM) ################

# Model specification:

## Specify the design matrix for the model:
design.matrix <- designMatrices(resp=cesd.responses, modeltype="RSM", constraint = "items")$A

## Run the RSM using the design matrix
RSM.CESD_JMLE <- tam.jml(cesd.responses, A = design.matrix, constraint = "items",
                         control=list(maxiter=500), version=2)

# Rating Scale Analysis with the RSM:

# A.	Average examinee locations within rating scale categories

#* Note: Average examinee location estimates should be non-decreasing as categories increase

## Store person parameters and their standard errors in a dataframe object:
person.locations_JMLE <- cbind.data.frame(cesd$id,
                                          RSM.CESD_JMLE$theta,
                                          RSM.CESD_JMLE$errorWLE)

names(person.locations_JMLE) <- c("id", "theta", "SE")

## Overall summary of person location estimates:
summary(person.locations_JMLE)

## Calculate the average participant location estimate within categories across items:

### Create empty objects in which to store the results for each category:
zero <- NULL
one <- NULL
two <- NULL
three <- NULL

### Use a for-loop to add results to the objects:
for(item.number in 1:ncol(cesd.responses)){
  
  item <- cbind.data.frame(cesd$id, cesd.responses[, item.number])
  names(item) <- c("id", "resp")
  cat.0 <- subset(item, item$resp == 0)
  cat.1 <- subset(item, item$resp == 1)
  cat.2 <- subset(item, item$resp == 2)
  cat.3 <- subset(item, item$resp == 3)
  
  merged.cat.0 <- merge(cat.0, person.locations_JMLE, by = "id")
  zero[item.number] <- mean(merged.cat.0$theta)
  
  merged.cat.1 <- merge(cat.1, person.locations_JMLE, by = "id")
  one[item.number] <- mean(merged.cat.1$theta)
  
  merged.cat.2 <- merge(cat.2, person.locations_JMLE, by = "id")
  two[item.number] <- mean(merged.cat.2$theta)
  
  merged.cat.3 <- merge(cat.3, person.locations_JMLE, by = "id")
  three[item.number] <- mean(merged.cat.3$theta)
}

### Calculate average particpant locations for each category:
mean(zero)
mean(one)
mean(two)
mean(three)



# B.	Logit-scale location estimates of rating scale category thresholds

#* Note: Threshold locations should be non-decreasing as categories increase

### Specify the number of thresholds as the maximum observed score in the response matrix (be sure the responses begin at category 0):
n.thresholds <- max(cesd.responses)

### Calculate adjacent-category threshold values:
tau.estimates_JMLE <- NULL

for(tau in 1: (n.thresholds - 1)){
  tau.estimates_JMLE[tau] <- RSM.CESD_JMLE$item1$xsi[ncol(cesd.responses) -1 + tau ]
}
tau.estimates_JMLE[n.thresholds] <- -(sum(tau.estimates_JMLE))

### Print adjacent-categories threshold estimates to the console:
tau.estimates_JMLE



# C.	Ordering of category probability curves

#* Note: Probabilities for ratings in lower categories should be highest for participants with
#* the lowest latent variable locations and lowest for participants with the highest latent 
#* variable locations 

plot(RSM.CESD_JMLE, type="items")


# D.	Distance between threshold location estimates on the logit scale

#* Note: Absolute distances between adjacent-categories threshold estimates should be 
#* large enough to identify examinees with meaningful differences on the latent variable 
#* and small enough to provide appropriate distinction among examinees

abs(tau.estimates_JMLE[2] - tau.estimates_JMLE[1])

abs(tau.estimates_JMLE[3] - tau.estimates_JMLE[2])



# E.	Distinct category probability curves

#* Note: The curve associated with each category should have a unique range of 
#* locations on the logit scale at which it is most probable

plot(RSM.CESD_JMLE, type="items")



# F.	Graphical model-data fit displays for categories

#* Note: There should be a close match between expected and observed 
#* ratings within individual rating scale categories

plot(RSM.CESD_JMLE, type="expected", low=-3, high=3, ngroups=6,
     groups_by_item=FALSE, wle=NULL, export=TRUE, export.type="png",
     export.args=list(), observed=TRUE, overlay=FALSE,
     ask=FALSE,  package="lattice", fix.devices=TRUE, nnodes=100)




############# Partial Credit Model (PCM) ################

# Model specification:

## Specify the design matrix for the model:
design.matrix <- designMatrices(resp=cesd.responses, modeltype="PCM", constraint = "items")$A

## Run the PCM using the design matrix
PCM.CESD_JMLE <- tam.jml(cesd.responses, A = design.matrix, constraint = "items",
                         control=list(maxiter=500), version=2, verbose = FALSE )


# Examine item results:

### Extract the item location estimates from the model
items_JMLE <- PCM.CESD_JMLE$item$xsi.item

### Center the item parameter estimates for ease of interpretation
uncentered.item.locations_JMLE <- items_JMLE
centered.item.locations_JMLE <- scale(uncentered.item.locations_JMLE, scale = FALSE)

### Request summary statistics for the centered item estimates
summary(centered.item.locations_JMLE)

# Rating Scale Analysis with the PCM:

# A.	Average examinee locations within rating scale categories

#* Note: Average examinee location estimates should be non-decreasing as 
#* categories increase for individual items

## Store person parameters and their standard errors in a dataframe object:
person.locations_JMLE <- cbind.data.frame(cesd$id,
                                          PCM.CESD_JMLE$theta,
                                          PCM.CESD_JMLE$errorWLE)

names(person.locations_JMLE) <- c("id", "theta", "SE")

### Subtract the original (uncentered) item mean location from the person locations:
person.locations_JMLE$theta_adjusted <- person.locations_JMLE$theta - mean(uncentered.item.locations_JMLE)

### Overall summary of person location estimates:
summary(person.locations_JMLE)

## Create a matrix in which to store average theta estimates for each item-specific category:
category_thetas <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = n.thresholds + 1)

### Use a for-loop to add the results to the matrix:
for(item.number in 1:ncol(cesd.responses)){
  
  item <- cbind.data.frame(cesd$id, cesd.responses[, item.number])
  
  names(item) <- c("id", "resp")
  cat.0 <- subset(item, item$resp == 0)
  cat.1 <- subset(item, item$resp == 1)
  cat.2 <- subset(item, item$resp == 2)
  cat.3 <- subset(item, item$resp == 3)
  
  merged.cat.0 <- merge(cat.0, person.locations_JMLE, by = "id")
  category_thetas[item.number, 1] <- mean(merged.cat.0$theta_adjusted)
  
  merged.cat.1 <- merge(cat.1, person.locations_JMLE, by = "id")
  category_thetas[item.number, 2] <- mean(merged.cat.1$theta_adjusted)
  
  merged.cat.2 <- merge(cat.2, person.locations_JMLE, by = "id")
  category_thetas[item.number, 3] <- mean(merged.cat.2$theta_adjusted)
  
  merged.cat.3 <- merge(cat.3, person.locations_JMLE, by = "id")
  category_thetas[item.number, 4] <- mean(merged.cat.3$theta_adjusted)
}

### Label the results:
category_thetas <- cbind.data.frame(colnames(cesd.responses), category_thetas)
names(category_thetas) <- c("item", "x = 0", "x = 1", "x = 2", "x = 3") 

### View the results:
category_thetas


# B.	Logit-scale location estimates of rating scale category thresholds

#* Note: Threshold locations for individual items should be non-decreasing as categories increase

## Calculate adjacent-category threshold values:

### Save the threshold estimates, which begin in column 5 of the item table:
tau.estimates_JMLE <- PCM.CESD_JMLE$item[, c(5 : (4+n.thresholds))]

### Specify the number of items that were included in the analysis:
n.items <- ncol(cesd.responses)

### Specify the number of thresholds as the maximum observed score in the response matrix (be sure the responses begin at category 0):
n.thresholds <- max(cesd.responses)

### Create a matrix in which to store the adjacent-category threshold values for each item:
tau.matrix_JMLE <- matrix(data = NA, ncol = n.thresholds, nrow = n.items)

### Calculate & add threshold values to the matrix:
for(item.number in 1:n.items){
  for(tau in 1:n.thresholds){
    tau.matrix_JMLE[item.number, tau] <- ifelse(tau == 1,
                                                (tau.estimates_JMLE[item.number, tau] -  
                                                   uncentered.item.locations_JMLE[item.number]),
                                                
                                                (tau.estimates_JMLE[item.number, tau] -
                                                   sum(tau.estimates_JMLE[item.number, c((tau-1))]) - 
                                                   uncentered.item.locations_JMLE[item.number]))
  }
}


## View the threshold estimates:
tau.matrix_JMLE


# C.	Ordering of category probability curves

#* Note: Probabilities for ratings in lower categories should be highest for 
#* participants with the lowest latent variable locations and lowest for 
#* participants with the highest latent variable locations for individual items

plot(PCM.CESD_JMLE, type="items")



# D.	Distance between threshold location estimates on the logit scale

#* Note: Absolute distances between adjacent-categories threshold estimates for 
#* individual items should be large enough to identify examinees with meaningful differences 
#* on the latent variable and small enough to provide appropriate distinction among examinees


## Create a matrix in which to save the distance between adjacent threshold estimates:
threshold_distances <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = (n.thresholds - 1))

## Use a for-loop to add results to the matrix:
for(item.number in 1:(ncol(cesd.responses))){
  threshold_distances[item.number, 1] <- abs(tau.matrix_JMLE[item.number, 2] - tau.matrix_JMLE[item.number, 1])
  threshold_distances[item.number, 2] <- abs(tau.matrix_JMLE[item.number, 3] - tau.matrix_JMLE[item.number, 2])
}

## Add labels to the matrix:
threshold_distances <- as.data.frame(threshold_distances)
names(threshold_distances) <- c("|tau 2 - tau 1|", "|tau 3 - tau 2|")

## View the results:
threshold_distances


# E.	Distinct category probability curves

#* Note: The curve associated with each category should have a unique range of locations 
#* on the logit scale at which it is most probable for individual items.

plot(PCM.CESD_JMLE, type="items")


# F.	Graphical model-data fit displays for categories

#* Note: There should be a close match between expected and observed 
#* ratings within individual rating scale categories for individual items

plot(PCM.CESD_JMLE, type="expected", low=-3, high=3, ngroups=6,
     groups_by_item=FALSE, wle=NULL, export=TRUE, export.type="png",
     export.args=list(), observed=TRUE, overlay=FALSE,
     ask=FALSE,  package="lattice", fix.devices=TRUE, nnodes=100)



############# Partial-Credit Many-Facet Rasch Model (PC-MFR) ################

# Model specification:

## specify facets that will be included in the model
### (Note that items are a facet by default)

facets <- cesd[, "education", drop=FALSE] 

## Specify person identification numbers:
pid <- cesd$id 

## Specify the response matrix
resp <- cesd.responses

## Specify the model formula:
PC_MFR_formula <- ~ item + education + (step+education:step)

# Run the PC-MFR model:
PC_MFR_model <- tam.mml.mfr(resp= resp, facets = facets,
                            formulaA= PC_MFR_formula,
                            pid=pid, verbose = FALSE, 
                            constraint = "items")

## Request a summary of the model results:
summary(PC_MFR_model)

### Examine Facet Estimates:

facet.estimates <- PC_MFR_model$xsi.facets # all facets together

item.estimates <- subset(facet.estimates, facet.estimates$facet == "item")

education.estimates <- subset(facet.estimates, facet.estimates$facet == "education")

threshold.estimates <- subset(facet.estimates, facet.estimates$facet == "step")

education_threshold.estimates <- subset(facet.estimates, facet.estimates$facet == "step:education")


### Examine item locations:

summary(item.estimates$xsi)
hist(item.estimates$xsi, xlab = "Logits", col = "darkseagreen3", 
     main = "Item Location Estimates")

### Examine education locations:

summary(education.estimates$xsi)
plot(education.estimates$xsi, main = "Education Subgroup Locations",
     xlab = "Education Subgroup Number", ylab = "Location Estimate", pch = 18)

### Examine person locations:

person.estimates <- tam.wle(PC_MFR_model)
theta <- person.estimates$theta
summary(theta)
hist(theta, xlab = "Logits", col = "darkseagreen3", main = "Person Location Estimates")


### Store person parameters and their standard errors in a dataframe object:
person.locations_PCMFR <- cbind.data.frame(cesd$id,
                                          person.estimates$theta,
                                          person.estimates$error)

names(person.locations_PCMFR) <- c("id", "theta", "SE")


# Rating Scale Analysis with the PC-MFRM:

# A.	Average examinee locations within rating scale categories

#* Note: Average examinee location estimates should be non-decreasing as 
#* categories increase for individual items


## Create a matrix in which to store average theta estimates for each group-specific category:
ngroups <- length(unique(education.estimates$parameter))
n.thresholds <- max(cesd.responses)
n.items <- ncol(cesd.responses)
group_category_thetas <- matrix(data = NA, nrow = ngroups, ncol = n.thresholds + 1)


### Use a for-loop to add the results to the matrix:
for(group.number in 1:ngroups){
  
  group <- subset(cesd, cesd$education == group.number)
  group <- subset(group, select = c(id, item1:item20))
  
  category_thetas <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = n.thresholds + 1)
  
  item.number <- 1
  for(item.number in 1:n.items){
    group.item <- group[, c(1, (item.number + 1))]
  
    cat.0 <- subset(group.item, group.item[, 2] == 0)
    cat.1 <- subset(group.item, group.item[, 2] == 1)
    cat.2 <- subset(group.item, group.item[, 2] == 2)
    cat.3 <- subset(group.item, group.item[, 2] == 3)
  
    merged.cat.0 <- merge(cat.0, person.locations_PCMFR, by = "id")
    category_thetas[item.number, 1] <- mean(merged.cat.0$theta)
  
    merged.cat.1 <- merge(cat.1, person.locations_PCMFR, by = "id")
    category_thetas[item.number, 2] <- mean(merged.cat.1$theta)
  
    merged.cat.2 <- merge(cat.2, person.locations_PCMFR, by = "id")
    category_thetas[item.number, 3] <- mean(merged.cat.2$theta)
  
    merged.cat.3 <- merge(cat.3, person.locations_PCMFR, by = "id")
    category_thetas[item.number, 4] <- mean(merged.cat.3$theta)
}

  group_category_thetas[group.number, 1] <- mean(category_thetas[,1], na.rm = TRUE)
  group_category_thetas[group.number, 2] <- mean(category_thetas[,2], na.rm = TRUE)
  group_category_thetas[group.number, 3] <- mean(category_thetas[,3], na.rm = TRUE)
  group_category_thetas[group.number, 4] <- mean(category_thetas[,4], na.rm = TRUE)
}

### Label the results:
group.names <- education.estimates$parameter
group_category_thetas <- cbind.data.frame(group.names, group_category_thetas)
names(group_category_thetas) <- c("education level", "x = 0", "x = 1", "x = 2", "x = 3") 

### View the results:
group_category_thetas


# B.	Logit-scale location estimates of rating scale category thresholds

#* Note: Threshold locations for individual elements
#* should be non-decreasing as categories increase

## First, put group-specific thresholds into a matrix:

### Create a matrix in which to store threshold estimates for each education group:
ngroups <- length(unique(cesd$education))
n.thresholds <- max(cesd.responses)
group_taus <- matrix(data = NA, nrow = ngroups, ncol = n.thresholds)

### re-structure the threshold estimates so that they can be added to the matrix:
all_group_taus <- t(education_threshold.estimates$xsi)

### Use a for-loop to add the results to the matrix. 

#* Note: For group 1, the thresholds for the first group are equal to the overall 
#* threshold estimates from the model. We will store the group-specific threshold
#* locations as zero logits for now, and then add the threshold values later.

group_taus[1, ] <- rep(0, n.thresholds)

for(group.number in 2:ngroups){
  group_taus[group.number,] <- all_group_taus[c((group.number * n.thresholds) - (n.thresholds - 1)) :
                                              (group.number * n.thresholds)]
}

### convert the matrix to a data frame for easier manipulation and labeling:
group_taus <- as.data.frame(group_taus)

### label the education-specific threshold estimates:
names(group_taus) <- c("t1", "t2", "t3")

### add overall threshold estimates:
group_taus$t1 <- group_taus$t1 + threshold.estimates$xsi[1]
group_taus$t2 <- group_taus$t2 + threshold.estimates$xsi[2]
group_taus$t3 <- group_taus$t3 + threshold.estimates$xsi[3]


### View the results:
group_taus

# C.	Ordering of category probability curves

#* Note: Probabilities for ratings in lower categories should be highest for participants with the 
#* #* lowest latent variable locations and lowest for participants with the highest latent variable locations 
#* for elements within explanatory facets (e.g., education subgroups)

plot(PC_MFR_model, type="items")

# D.	Distance between threshold location estimates on the logit scale

#* Note: Absolute distances between adjacent-categories threshold estimates should be 
#* large enough to identify examinees with meaningful differences on the latent variable 
#* and small enough to provide appropriate distinction among examinees 
#* for elements within explanatory facets (e.g., education subgroups)

## Create a matrix in which to save the distance between adjacent threshold estimates
threshold_distances <- matrix(data = NA, nrow = ngroups, ncol = (n.thresholds - 1))

## Use a for-loop to add results to the matrix
for(group.number in 1:ngroups){
  threshold_distances[group.number, 1] <- abs(group_taus[group.number, 2] - group_taus[group.number, 1])
  threshold_distances[group.number, 2] <- abs(group_taus[group.number, 3] - group_taus[group.number, 2])
}

## Add labels to the matrix:
threshold_distances <- cbind.data.frame(group.names, threshold_distances)
names(threshold_distances) <- c("education level", "|tau 2 - tau 1|", "|tau 3 - tau 2|")

## View the results:
threshold_distances


# E.	Distinct category probability curves

#* Note: The curve associated with each category should have a unique range of locations 
#* on the logit scale at which it is most probable for elements within explanatory facets 
#* (e.g., education subgroups)

plot(PC_MFR_model, type="items")


# F.	Graphical model-data fit displays for categories

#* Note: There should be a close match between expected and observed ratings 
#* within individual rating scale categories for elements of explanatory facet(s)

plot(PC_MFR_model, type="expected", low=-3, high=3, ngroups=6,
     groups_by_item=FALSE, wle=NULL, export=TRUE, export.type="png",
     export.args=list(), observed=TRUE, overlay=FALSE,
     ask=FALSE,  package="lattice", fix.devices=TRUE, nnodes=100)


