#------------------- Code to Support Chapter 4 --------------------#

########## Prepare for the Analysis ###############

# Install and load the TAM package:
install.packages("TAM")
library("TAM")

# Install and load the ltm package
install.packages("ltm")
library("ltm")

# Install and load the mokken package
install.packages("mokken")
library("mokken")

# Import the data:
#* Note: Set the working directory to the folder where you have saved the data.
#* Then, you should be able to import it using the code below:
cesd <- read.csv("Simulated_CESD_with_demographics.csv")

# Isolate the response matrix for analysis:
cesd.responses <- subset(cesd, select = c(item1:item20))

############ Generalized Partial Credit Model (GPCM) ############

# Estimate the GPCM:
cesd_gpcm <- TAM::tam.mml.2pl(cesd.responses, irtmodel="GPCM", verbose = FALSE)

# Request a summary of the model results:
summary(cesd_gpcm)

# Global model fit:

## estimate PCM for comparison purposes:

### Run the PCM:
cesd_pcm <- tam.mml(cesd.responses, constraint = "items", verbose = FALSE )

## Global fit analysis for the PCM:
global.fit_pcm <- tam.modelfit.IRT(cesd_pcm)
global.fit_pcm$modelfit.test

## Global fit analysis for the GPCM:
global.fit_gpcm <- tam.modelfit.IRT(cesd_gpcm, progress=TRUE )
global.fit_gpcm$modelfit.test

# Compare global fit between the PCM and GPCM:
compare.models <- CDM::IRT.compareModels(cesd_pcm, cesd_gpcm )
compare.models$LRtest


# Chi-square tests of conditional independence for item pairs:
item.fit_gpcm <- as.data.frame(global.fit_gpcm$chi2.stat)
summary(item.fit_gpcm$p)
hist(item.fit_gpcm$p)

item.fit_gpcm2 <- global.fit_gpcm$stat.itempair
c2 <- global.fit_gpcm$chi2.stat

?tam.modelfit.IRT



# Examine item parameters
item.parameters_GPCM <- cesd_gpcm$item_irt
item.parameters_GPCM
summary(item.parameters_GPCM)

# Examine person parameters
person.locations.estimate_GPCM <- tam.wle(cesd_gpcm)

## Store person parameters and their standard errors in a dataframe object:
person.locations_GPCM <- cbind.data.frame(person.locations.estimate_GPCM$pid,
                                          person.locations.estimate_GPCM$theta,
                                          person.locations.estimate_GPCM$error)
names(person.locations_GPCM) <- c("id", "theta", "SE")

### View summary statistics for person parameters:
summary(person.locations_GPCM)

hist(person.locations_GPCM$theta, main = "GPCM Person Location Estimates: CESD", 
     xlab = "Theta", col = "darkseagreen3")

# Rating Scale Analysis for the GPCM

# A.	Average examinee locations within rating scale categories
#* Note: Average examinee location estimates should be non-decreasing as categories increase for individual items

## Specify the number of thresholds as the maximum observed score in the response matrix (be sure the responses begin at category 0):
n.thresholds <- max(cesd.responses)

### Create a matrix in which to store average theta estimates for each item-specific category:
category_thetas <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = n.thresholds + 1)

### Use a for-loop to add the results to the matrix:
for(item.number in 1:ncol(cesd.responses)){
  item <- cbind.data.frame(cesd$id, cesd.responses[, item.number])
  
  names(item) <- c("id", "resp")
  cat.0 <- subset(item, item$resp == 0)
  cat.1 <- subset(item, item$resp == 1)
  cat.2 <- subset(item, item$resp == 2)
  cat.3 <- subset(item, item$resp == 3)
  
  merged.cat.0 <- merge(cat.0, person.locations_GPCM, by = "id")
  category_thetas[item.number, 1] <- mean(merged.cat.0$theta)
  
  merged.cat.1 <- merge(cat.1, person.locations_GPCM, by = "id")
  category_thetas[item.number, 2] <- mean(merged.cat.1$theta)
  
  merged.cat.2 <- merge(cat.2, person.locations_GPCM, by = "id")
  category_thetas[item.number, 3] <- mean(merged.cat.2$theta)
  
  merged.cat.3 <- merge(cat.3, person.locations_GPCM, by = "id")
  category_thetas[item.number, 4] <- mean(merged.cat.3$theta)
}

### Label the results:
category_thetas <- cbind.data.frame(colnames(cesd.responses), category_thetas)
names(category_thetas) <- c("item", "x = 0", "x = 1", "x = 2", "x = 3") 

### View the results:
category_thetas

## B.	Logit-scale location estimates of rating scale category thresholds
#* Note: Threshold locations for individual items should be non-decreasing as categories increase 

### Specify the number of thresholds as the maximum observed score in the response matrix (be sure the responses begin at category 0):
n.thresholds <- max(cesd.responses)

### Specify the number of items in the response matrix:
n.items <- ncol(cesd.responses)

### Create a matrix in which to store the adjacent-category threshold values for each item:
tau.matrix_GPCM <- matrix(data = NA, ncol = n.thresholds, nrow = n.items)

### Calculate & add threshold values to the matrix:
for(item.number in 1:n.items){
  for(tau in 1:n.thresholds){
    tau.matrix_GPCM[item.number, tau] <- item.parameters_GPCM$beta[item.number] +
      item.parameters_GPCM[item.number, 3 + tau]
  }
}

### View the threshold estimates:
tau.matrix_GPCM

### View standard errors for thresholds:
cat.se <- cesd_gpcm$se.AXsi


# C.	Ordering of category probability curves

#* Note: Probabilities for ratings in lower categories should be highest for participants with the 
#* #* lowest latent variable locations and lowest for participants with the highest latent variable locations for individual items

plot(cesd_gpcm, type="items")

# D.	Distance between threshold location estimates on the logit scale

#* Note: Absolute distances between adjacent-categories threshold estimates should be large enough to 
#* identify examinees with meaningful differences on the latent variable and small enough 
#* to provide appropriate distinction among examinees for individual items 

## Create a matrix in which to save the distance between adjacent threshold estimates
threshold_distances <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = (n.thresholds - 1))

## Use a for-loop to add results to the matrix
for(item.number in 1:(ncol(cesd.responses))){
  threshold_distances[item.number, 1] <- abs(tau.matrix_GPCM[item.number, 2] - tau.matrix_GPCM[item.number, 1])
  threshold_distances[item.number, 2] <- abs(tau.matrix_GPCM[item.number, 3] - tau.matrix_GPCM[item.number, 2])
}

## Add labels to the matrix:
threshold_distances <- as.data.frame(threshold_distances)
names(threshold_distances) <- c("|tau 2 - tau 1|", "|tau 3 - tau 2|")

## View the results:
threshold_distances
summary(threshold_distances)


# E.	Distinct category probability curves

#* Note: The curve associated with each category should have a unique range of locations 
#* on the logit scale at which it is most probable for individual items.

## Use a for-loop to generate plots of item-specific category probability curves
n.items <- ncol(cesd.responses)
for(item.number in 1:n.items){
  p <- plot(cesd_gpcm, type="items", items = item.number)
  
  cats <- t(as.data.frame(p$res_items$probs_plot[item.number]))

  plot(x = p$res_items$theta, y = cats[,1], pch = NA,
       ylab = "Category Probability",
       xlab = "Theta", ylim = c(0, 1), xlim = c(min(p$res_items$theta), max(p$res_items$theta)),
       main = paste("Category Probabilities for CESD Item", item.number, "\n(Generalized Partial Credit Model)"))
  lines(x = p$res_items$theta, y = cats[,1], col = "red", lty = 1, lwd = 2)
  lines(x = p$res_items$theta, y = cats[,2], col = "dark green", lty = 2, lwd = 2)
  lines(x = p$res_items$theta, y = cats[,3], col = "blue", lty = 3, lwd = 2)
  lines(x = p$res_items$theta, y = cats[,4], col = "purple", lty = 4, lwd = 2)
  
  legend("topright", c("P(x=0)", "P(x=1)", "P(x=2)", "P(x=3)"),
         col=c("red", "dark green", "blue", "purple"),
         lty = c(1, 2, 3, 4), lwd = c(2, 2, 2, 2))
}


############ Graded Response Model (GRM) ############

library(ltm)

#* Note: The ltm package assumes that lower ratings indicate higher levels of the construct
#* As a result, we will reverse-code the CESD responses prior to analysis with the GRM:
cesd.responses.r <- 3 - cesd.responses

# Estimate the GRM:
cesd_GRM <- grm(cesd.responses.r, Hessian = TRUE, IRT.param = TRUE)

## Request a summary of the model results:
summary(cesd_GRM, robust.se = TRUE)

# Global model fit
cesd_GRM$log.Lik

# Item fit
item.fit_grm <- margins(cesd_GRM)
item.fit_grm


# Examine item parameters
item.parameters_GRM <- coef.grm(cesd_GRM)
item.parameters_GRM


# Examine standard errors:
model_summary <- summary(cesd_GRM, robust.se = TRUE)

model_summary

# examine item discrimination:

## store the parameters in a vector
alpha <- NULL

for(item.number in 1:20){
  i <- as.data.frame(model_summary$coefficients[item.number])
  alpha[item.number] <- i[4,1]
}

## view & summarize item discrimination estimates:
alpha
summary(alpha)
sd(alpha)

# examine overall item location estimates:

## store the parameters in a vector
delta <- NULL

for(item.number in 1:20){
  i <- as.data.frame(item.parameters_GRM[item.number,])
  delta[item.number] <- mean(i[c(1:3),])
}

## view & summarize item location estimates:
delta
summary(delta)
sd(delta)

# Examine person parameters

## estimate person locations:
person.locations.estimate_GRM <- factor.scores(cesd_GRM)

## view person location estimates in a histogram
graphics.off()
hist(person.locations.estimate_GRM$score.dat$z1,
     main = "GRM Person Location Estimates: CESD", 
     xlab = "Theta", col = "darkseagreen3")

#* Note: In the ltm package, person locations are estimated using groups of 
#* persons who have similar score patterns. Person estimates are not reported
#* for individual persons from the original score data.

## View summary statistics for person parameters:
summary(person.locations.estimate_GRM$score.dat$z1)
## Summary statistics for person standard errors:
summary(person.locations.estimate_GRM$score.dat$se.z1)

# Rating Scale Analysis for the GRM

# A.	Average examinee locations within rating scale categories

#* Note: Average examinee location estimates should be non-decreasing as categories 
#* increase for individual items

## Create a matrix in which to store the results:
category_thetas <- matrix(data = NA, nrow = ncol(cesd.responses.r), ncol = n.thresholds + 1)

## Add results to the matrix:
for(item.number in 1:ncol(cesd.responses)){
 item <- cbind.data.frame(person.locations.estimate_GRM$score.dat[, item.number],
                           person.locations.estimate_GRM$score.dat$z1)
  
  names(item) <- c("resp", "theta")
  cat.0 <- subset(item, item$resp == 1)
  cat.1 <- subset(item, item$resp == 2)
  cat.2 <- subset(item, item$resp == 3)
  cat.3 <- subset(item, item$resp == 4)
  
  category_thetas[item.number, 1] <- mean(cat.0$theta)
  category_thetas[item.number, 2] <- mean(cat.1$theta)
  category_thetas[item.number, 3] <- mean(cat.2$theta)
  category_thetas[item.number, 4] <- mean(cat.3$theta)
}

## View the results:
category_thetas


# B.	Distance between threshold location estimates on the logit scale

#* Note: Absolute value of the differences between item-specific cumulative 
#* rating scale category threshold estimates should be large enough to identify examinees 
#* with meaningful differences on the latent variable and small enough to provide 
#* appropriate distinction among examinees for individual items.

thresholds <- item.parameters_GRM[, -4]

## Calculate the distance between adjacent categories and store the results in a matrix:
n.thresholds <- max(cesd.responses)

threshold_distances <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = (n.thresholds - 1))

for(item.number in 1:(ncol(cesd.responses))){
  threshold_distances[item.number, 1] <- abs(thresholds[item.number, 2] - thresholds[item.number, 1])
  threshold_distances[item.number, 2] <- abs(thresholds[item.number, 3] - thresholds[item.number, 2])
}

## Add labels to the matrix:
threshold_distances <- as.data.frame(threshold_distances)

names(threshold_distances) <- c("|tau 2 - tau 1|", "|tau 3 - tau 2|")

## View & summarize the results:
threshold_distances

summary(threshold_distances)


# C.	Plots of item-specific cumulative category probabilities

#* Note: Cumulative category probabilities for a given item 
#* should be distinct (i.e., no overlap) for individual items

## Use a for-loop to generate plots of category probabilities for all items:
for(item.number in 1:ncol(cesd.responses)){
  p <- plot(cesd_GRM, type = "OCCu", items = item.number, plot = FALSE)

  cats <- as.data.frame(p$pr[1])
  
  plot(p$z, cats[,1], pch = NA, ylab = "Cumulative Category Probability",
       xlab = "Theta", ylim = c(0, 1), main = paste("Cumulative Category Probabilities for CESD Item", item.number, "\n(Graded Response Model)"))
  lines(x = p$z, y = cats[,1], col = "red", lty = 1, lwd = 2)
  lines(x = p$z, y = cats[,2], col = "dark green", lty = 2, lwd = 2)
  lines(x = p$z, y = cats[,3], col = "blue", lty = 3, lwd = 2)

  arrows(x0 = thresholds[item.number, 1], y0 = 0, x1 = thresholds[item.number, 1], y1 = .5, length =  0,
         lty = 1, lwd = 1, col = "black")
  arrows(x0 = thresholds[item.number, 2] , y0 = 0, x1 = thresholds[item.number, 2] , y1 = .5, length =  0,
         lty = 2, lwd = 1, col = "black")
  arrows(x0 = thresholds[item.number, 3] , y0 = 0, x1 = thresholds[item.number, 3] , y1 = .5, length =  0,
         lty = 3, lwd = 1, col = "black")
  legend("right", c("x>=1", "x>=2", "x>=3", 
                    "delta_i_1","delta_i_2","delta_i_3"),
         lty = c(1, 2, 3, 1, 2, 3),
         lwd = c(2, 2, 2, 1, 1, 1),
         col = c("red", "dark green", "blue", "black", "black", "black", "black"))
  }



# D. Plots of item-specific individual category probabilities

#* Note: Curves associated with each category should have a distinct range of locations 
#* on the latent variable at which it is the most probable for individual items


## Use a for-loop to generate plots of category probabilities for all items:
for(item.number in 1:ncol(cesd.responses)){
  p <- plot(cesd_GRM, type = "ICC", items = item.number, plot = FALSE)

  cats <- as.data.frame(p$pr[1])
  
  plot(x = p$z, y = cats[,1], pch = NA, ylab = "Category Probability",
       xlab = "Theta", ylim = c(0, 1), main = paste("Category Probabilities for CESD Item", item.number, "\n(Graded Response Model)"))
  lines(x = p$z, y = cats[,1], col = "red", lty = 1, lwd = 2)
  lines(x = p$z, y = cats[,2], col = "dark green", lty = 2, lwd = 2)
  lines(x = p$z, y = cats[,3], col = "blue", lty = 3, lwd = 2)
  lines(x = p$z, y = cats[,4], col = "purple", lty = 4, lwd = 2)
  
  arrows(x0 = thresholds[item.number, 1], y0 = 0, x1 = thresholds[item.number, 1], y1 = .5, length =  0,
         lty = 1, lwd = 1, col = "black")
  arrows(x0 = ((thresholds[item.number, 2] + thresholds[item.number, 1])/2), y0 = 0, x1 = ((thresholds[item.number, 2] + thresholds[item.number, 1])/2), y1 = max(cats[, 2]), length =  0,
         lty = 2, lwd = 1, col = "black")
  arrows(x0 = ((thresholds[item.number, 3] + thresholds[item.number, 2])/2), y0 = 0, x1 = ((thresholds[item.number, 3] + thresholds[item.number, 2])/2), y1 = max(cats[, 3]), length =  0,
         lty = 3, lwd = 1, col = "black")
  arrows(x0 = thresholds[item.number, 3], y0 = 0, x1 = thresholds[item.number, 3], y1 = .5, length =  0,
         lty = 4, lwd = 1, col = "black", pch = 1)
  
  legend("right", c("x=0", "x=1", "x=2", "x=3", 
                    "P(x=0)=0.5","mode(x=1)","mode(x=2)", "P(x=3)=0.5"),
         lty = c(1, 2, 3, 4,  1, 2, 3, 4),
         lwd = c(2, 2, 2, 1, 1, 1, 1, 1),
         col = c("red", "dark green", "blue", "purple", "black", "black", "black", "black"))
  }



############ Mokken Scale Analysis (MSA) ############

# Examine overall MSA model fit:

## Monotone Homogeneity Model

### Evaluate monotonicity:
mono.results <- check.monotonicity(cesd.responses)

### Numeric monotonicity results:
summary(mono.results)

### Graphical monotonicity results:
plot(mono.results, curves = "both", ask = FALSE)

### Evaluate scalability:
scalability <- coefH(cesd.responses)

### Overall scalability for the CES-D scale:
scalability$H

### Item-level scalability:
scalability$Hi

## Double Monotonicity Model

### Evaluate items for invariant item ordering (IIO):
iio.results <- check.iio(cesd.responses, method = "MIIO")

### Numeric summary of IIO results:
summary(iio.results)

## Graphical IIO results:
plot(iio.results, ask = FALSE,  color = c("dark green", "plum"))


# Rating Scale Analysis with MSA

# A. Average rest scores within categories

#* Note: Examinee restscores should be non-decreasing across increasing 
#* rating scale categories for individual items

## Calculate restscores:
cesd.responses_with_sum <- cesd.responses
cesd.responses_with_sum$sum_score <- rowSums(cesd.responses_with_sum)

## Create a matrix in which to store the average restscores within categories for items:
n.thresholds <- max(cesd.responses)
n.cat <- n.thresholds + 1

category_rest_scores <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = n.cat)

for(item.number in 1:ncol(cesd.responses)){
  
  item <- cbind.data.frame(cesd.responses_with_sum[, item.number],
                           (cesd.responses_with_sum$sum_score - cesd.responses_with_sum[, item.number]))
  
  names(item) <- c("resp", "rest_score")
  cat.0 <- subset(item, item$resp == 0)
  cat.1 <- subset(item, item$resp == 1)
  cat.2 <- subset(item, item$resp == 2)
  cat.3 <- subset(item, item$resp == 3)
  
  category_rest_scores[item.number, 1] <- mean(cat.0$rest_score)
  category_rest_scores[item.number, 2] <- mean(cat.1$rest_score)
  category_rest_scores[item.number, 3] <- mean(cat.2$rest_score)
  category_rest_scores[item.number, 4] <- mean(cat.3$rest_score)
}

## Add labels to the matrix:
category_rest_scores <- cbind.data.frame(colnames(cesd.responses), category_rest_scores)
names(category_rest_scores) <- c("item", "x = 0", "x = 1", "x = 2", "x = 3") 

## View the results:
category_rest_scores

## B. Counts of Item & Category Violations of Monotonicity

#* Note: We are looking for adherence to monotonicity at the category level 
#* (infrequent and small violations of monotonicity) for individual items

## Evaluate monotonicity:
mono.results <- check.monotonicity(cesd.responses)

## View a numeric summary of the overall item monotonicity  results
summary(mono.results)

## Calculate the frequency of violations >= .03:
n.items <- ncol(cesd.responses)

### Create empty objects in which to store the frequencies of monotonicity violations
cat_1_violations <- NULL
cat_2_violations <- NULL
cat_3_violations <- NULL

### Use a for-loop to add results to the empty objects:
for(item.number in 1:n.items){
  
  # get monotonicity results:
  item <- as.data.frame(mono.results$results[[item.number]][[2]])
  
  # calculate restscore:
  cesd.ratings_with_sum <- cesd.responses
  cesd.ratings_with_sum$sum <- apply(cesd.ratings_with_sum[,c(1:20)], 1, sum)
  cesd.ratings_with_sum$restscore <- cesd.ratings_with_sum$sum - cesd.ratings_with_sum[, item.number]
  
  
  cat_mono_1 <- NULL
  cat_mono_2 <- NULL
  cat_mono_3 <- NULL
  
  for (rs_group in 1:(nrow(item)-1)){
    cat_mono_1[rs_group] <- ((item$`P(X >=1)`[rs_group + 1] >= item$`P(X >=1)`[rs_group]) |
                               (abs(item$`P(X >=1)`[rs_group] - item$`P(X >=1)`[rs_group + 1] ) < 0.03))
    
    cat_mono_2[rs_group] <- ((item$`P(X >=2)`[rs_group + 1] >= item$`P(X >=2)`[rs_group]) |
                               (abs(item$`P(X >=2)`[rs_group] - item$`P(X >=2)`[rs_group + 1] ) < 0.03))
    
    cat_mono_3[rs_group] <- ((item$`P(X >=3)`[rs_group + 1] >= item$`P(X >=3)`[rs_group]) |
                               (abs(item$`P(X >=3)`[rs_group] - item$`P(X >=3)`[rs_group + 1] ) < 0.03))
  }
  
  
  cat_1_violations[item.number] <- (nrow(item)-1) - sum(cat_mono_1)
  cat_2_violations[item.number] <- (nrow(item)-1) - sum(cat_mono_2)
  cat_3_violations[item.number]<- (nrow(item)-1) - sum(cat_mono_3)
}

### View count of violations per category per item:
cat_1_violations
cat_2_violations
cat_3_violations

### Save the results in a data frame object:
item_cat_frequency <- cbind.data.frame(cat_1_violations,
                                        cat_2_violations,
                                        cat_3_violations)

### View the results:
item_cat_frequency

summary(item_cat_frequency)

## Examine the magnitude of monotonicity violations for each cumulative category:
n.items <- ncol(cesd.responses)

### Create empty objects in which to store the magnitude of monotonicity violations
cat_1_violations <- NULL
cat_2_violations <- NULL
cat_3_violations <- NULL

### Use a for-loop to add results to the empty objects:
for(item.number in 1:n.items){
  
  # get monotonicity results:
  item <- as.data.frame(mono.results$results[[item.number]][[2]])
  
  # calculate restscore:
  cesd.ratings_with_sum <- cesd.responses
  cesd.ratings_with_sum$sum <- apply(cesd.ratings_with_sum[,c(1:20)], 1, sum)
  cesd.ratings_with_sum$restscore <- cesd.ratings_with_sum$sum - cesd.ratings_with_sum[, item.number]
  
  
  cat_mono_1 <- NULL
  cat_mono_2 <- NULL
  cat_mono_3 <- NULL
  
  for (rs_group in 1:(nrow(item)-1)){
    cat_mono_1[rs_group] <-ifelse((item$`P(X >=1)`[rs_group + 1] >= item$`P(X >=1)`[rs_group]),
                            NA, (abs(item$`P(X >=1)`[rs_group] - item$`P(X >=1)`[rs_group + 1] )))
    
    cat_mono_2[rs_group] <-ifelse((item$`P(X >=2)`[rs_group + 1] >= item$`P(X >=2)`[rs_group]),
                                  NA, (abs(item$`P(X >=2)`[rs_group] - item$`P(X >=2)`[rs_group + 1] )))
    
    cat_mono_3[rs_group] <-ifelse((item$`P(X >=3)`[rs_group + 1] >= item$`P(X >=3)`[rs_group]),
                                  NA, (abs(item$`P(X >=3)`[rs_group] - item$`P(X >=3)`[rs_group + 1] )))
    
  }
  
  
  cat_1_violations[item.number] <- ifelse((sum(cat_mono_1, na.rm = TRUE) > 0), max(cat_mono_1, na.rm = TRUE), 0)
  cat_2_violations[item.number] <- ifelse((sum(cat_mono_2, na.rm = TRUE) > 0), max(cat_mono_2, na.rm = TRUE), 0)
  cat_3_violations[item.number] <- ifelse((sum(cat_mono_3, na.rm = TRUE) > 0), max(cat_mono_3, na.rm = TRUE), 0)

}

### View count of violations per category per item:
cat_1_violations
cat_2_violations
cat_3_violations

### Save the results in a data frame object:
item_cat_violations <- cbind.data.frame(cat_1_violations,
                                        cat_2_violations,
                                        cat_3_violations)

### View the results:
item_cat_violations
summary(item_cat_violations)


# C, D, and E: Graphical displays of item-specific cumulative category monotonicity

#* Note: Plots of item-specific cumulative category probabilities should be 
#* non-decreasing over increasing restscores for individual items

plot(mono.results, curves = "both", color = "darkseagreen3", ask = FALSE)

