#------------------- Code to Support Chapter 5 --------------------#

########## Prepare for the Analysis ###############

# Install and load the mokken package
install.packages("mokken")
library("mokken")

# Import the data:
#* Note: Set the working directory to the folder where you have saved the data.
#* Then, you should be able to import it using the code below:
cesd <- read.csv("Simulated_CESD_with_demographics.csv")

# Isolate the response matrix for analysis:
cesd.responses <- subset(cesd, select = c(item1:item20))

############ Mokken Scale Analysis (MSA) ############

# Examine overall MSA model fit:

## Monotone Homogeneity Model

### Evaluate monotonicity:
mono.results <- check.monotonicity(cesd.responses)

### Numeric monotonicity results:
summary(mono.results)

### Graphical monotonicity results:
plot(mono.results, curves = "both", ask = FALSE)

### Evaluate scalability:
scalability <- coefH(cesd.responses)

### Overall scalability for the CES-D scale:
scalability$H

### Item-level scalability:
scalability$Hi

## Double Monotonicity Model

### Evaluate items for invariant item ordering (IIO):
iio.results <- check.iio(cesd.responses, method = "MIIO")

### Numeric summary of IIO results:
summary(iio.results)

## Graphical IIO results:
plot(iio.results, ask = FALSE,  color = c("dark green", "plum"))


# Rating Scale Analysis with MSA

# A. Average rest scores within categories

#* Note: Examinee restscores should be non-decreasing across increasing 
#* rating scale categories for individual items

## Calculate restscores:
cesd.responses_with_sum <- cesd.responses
cesd.responses_with_sum$sum_score <- rowSums(cesd.responses_with_sum)

## Create a matrix in which to store the average restscores within categories for items:
n.thresholds <- max(cesd.responses)
n.cat <- n.thresholds + 1

category_rest_scores <- matrix(data = NA, nrow = ncol(cesd.responses), ncol = n.cat)

for(item.number in 1:ncol(cesd.responses)){
  
  item <- cbind.data.frame(cesd.responses_with_sum[, item.number],
                           (cesd.responses_with_sum$sum_score - cesd.responses_with_sum[, item.number]))
  
  names(item) <- c("resp", "rest_score")
  cat.0 <- subset(item, item$resp == 0)
  cat.1 <- subset(item, item$resp == 1)
  cat.2 <- subset(item, item$resp == 2)
  cat.3 <- subset(item, item$resp == 3)
  
  category_rest_scores[item.number, 1] <- mean(cat.0$rest_score)
  category_rest_scores[item.number, 2] <- mean(cat.1$rest_score)
  category_rest_scores[item.number, 3] <- mean(cat.2$rest_score)
  category_rest_scores[item.number, 4] <- mean(cat.3$rest_score)
}

## Add labels to the matrix:
category_rest_scores <- cbind.data.frame(colnames(cesd.responses), category_rest_scores)
names(category_rest_scores) <- c("item", "x = 0", "x = 1", "x = 2", "x = 3") 

## View the results:
category_rest_scores

## B. Counts of Item & Category Violations of Monotonicity

#* Note: We are looking for adherence to monotonicity at the category level 
#* (infrequent and small violations of monotonicity) for individual items

## Evaluate monotonicity:
mono.results <- check.monotonicity(cesd.responses)

## View a numeric summary of the overall item monotonicity  results
summary(mono.results)

## Calculate the frequency of violations >= .03:
n.items <- ncol(cesd.responses)

### Create empty objects in which to store the frequencies of monotonicity violations
cat_1_violations <- NULL
cat_2_violations <- NULL
cat_3_violations <- NULL

### Use a for-loop to add results to the empty objects:
for(item.number in 1:n.items){
  
  # get monotonicity results:
  item <- as.data.frame(mono.results$results[[item.number]][[2]])
  
  # calculate restscore:
  cesd.ratings_with_sum <- cesd.responses
  cesd.ratings_with_sum$sum <- apply(cesd.ratings_with_sum[,c(1:20)], 1, sum)
  cesd.ratings_with_sum$restscore <- cesd.ratings_with_sum$sum - cesd.ratings_with_sum[, item.number]
  
  
  cat_mono_1 <- NULL
  cat_mono_2 <- NULL
  cat_mono_3 <- NULL
  
  for (rs_group in 1:(nrow(item)-1)){
    cat_mono_1[rs_group] <- ((item$`P(X >=1)`[rs_group + 1] >= item$`P(X >=1)`[rs_group]) |
                               (abs(item$`P(X >=1)`[rs_group] - item$`P(X >=1)`[rs_group + 1] ) < 0.03))
    
    cat_mono_2[rs_group] <- ((item$`P(X >=2)`[rs_group + 1] >= item$`P(X >=2)`[rs_group]) |
                               (abs(item$`P(X >=2)`[rs_group] - item$`P(X >=2)`[rs_group + 1] ) < 0.03))
    
    cat_mono_3[rs_group] <- ((item$`P(X >=3)`[rs_group + 1] >= item$`P(X >=3)`[rs_group]) |
                               (abs(item$`P(X >=3)`[rs_group] - item$`P(X >=3)`[rs_group + 1] ) < 0.03))
  }
  
  
  cat_1_violations[item.number] <- (nrow(item)-1) - sum(cat_mono_1)
  cat_2_violations[item.number] <- (nrow(item)-1) - sum(cat_mono_2)
  cat_3_violations[item.number]<- (nrow(item)-1) - sum(cat_mono_3)
}

### View count of violations per category per item:
cat_1_violations
cat_2_violations
cat_3_violations

### Save the results in a data frame object:
item_cat_frequency <- cbind.data.frame(cat_1_violations,
                                       cat_2_violations,
                                       cat_3_violations)

### View the results:
item_cat_frequency

summary(item_cat_frequency)

## Examine the magnitude of monotonicity violations for each cumulative category:
n.items <- ncol(cesd.responses)

### Create empty objects in which to store the magnitude of monotonicity violations
cat_1_violations <- NULL
cat_2_violations <- NULL
cat_3_violations <- NULL

### Use a for-loop to add results to the empty objects:
for(item.number in 1:n.items){
  
  # get monotonicity results:
  item <- as.data.frame(mono.results$results[[item.number]][[2]])
  
  # calculate restscore:
  cesd.ratings_with_sum <- cesd.responses
  cesd.ratings_with_sum$sum <- apply(cesd.ratings_with_sum[,c(1:20)], 1, sum)
  cesd.ratings_with_sum$restscore <- cesd.ratings_with_sum$sum - cesd.ratings_with_sum[, item.number]
  
  
  cat_mono_1 <- NULL
  cat_mono_2 <- NULL
  cat_mono_3 <- NULL
  
  for (rs_group in 1:(nrow(item)-1)){
    cat_mono_1[rs_group] <-ifelse((item$`P(X >=1)`[rs_group + 1] >= item$`P(X >=1)`[rs_group]),
                                  NA, (abs(item$`P(X >=1)`[rs_group] - item$`P(X >=1)`[rs_group + 1] )))
    
    cat_mono_2[rs_group] <-ifelse((item$`P(X >=2)`[rs_group + 1] >= item$`P(X >=2)`[rs_group]),
                                  NA, (abs(item$`P(X >=2)`[rs_group] - item$`P(X >=2)`[rs_group + 1] )))
    
    cat_mono_3[rs_group] <-ifelse((item$`P(X >=3)`[rs_group + 1] >= item$`P(X >=3)`[rs_group]),
                                  NA, (abs(item$`P(X >=3)`[rs_group] - item$`P(X >=3)`[rs_group + 1] )))
    
  }
  
  
  cat_1_violations[item.number] <- ifelse((sum(cat_mono_1, na.rm = TRUE) > 0), max(cat_mono_1, na.rm = TRUE), 0)
  cat_2_violations[item.number] <- ifelse((sum(cat_mono_2, na.rm = TRUE) > 0), max(cat_mono_2, na.rm = TRUE), 0)
  cat_3_violations[item.number] <- ifelse((sum(cat_mono_3, na.rm = TRUE) > 0), max(cat_mono_3, na.rm = TRUE), 0)
  
}

### View count of violations per category per item:
cat_1_violations
cat_2_violations
cat_3_violations

### Save the results in a data frame object:
item_cat_violations <- cbind.data.frame(cat_1_violations,
                                        cat_2_violations,
                                        cat_3_violations)

### View the results:
item_cat_violations
summary(item_cat_violations)


# C, D, and E: Graphical displays of item-specific cumulative category monotonicity

#* Note: Plots of item-specific cumulative category probabilities should be 
#* non-decreasing over increasing restscores for individual items

plot(mono.results, curves = "both", color = "darkseagreen3", ask = FALSE)

